@@ -2,13 +2,13 @@ package MooX::Types::MooseLike;
 use strict;
 use warnings FATAL => 'all';
 use Exporter 5.57 'import';
-our @EXPORT_OK = ();
-push @EXPORT_OK, 'exception_message';
+our @EXPORT_OK;
+push @EXPORT_OK, qw( exception_message inflate_type );
 use Module::Runtime qw(require_module);
 use Carp qw(confess croak);
 use List::Util qw(first);
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 sub register_types {
   my ($type_definitions, $into, $moose_namespace) = @_;
@@ -55,15 +55,23 @@ sub make_type {
     confess $type_definition->{message}->(@_) ;  ## no critic qw(ErrorHandling::RequireUseOfExceptions)
     };
 
-  my $full_name =
-    $moose_namespace
-    ? "${moose_namespace}::" . $type_definition->{name}
-    : $type_definition->{name};
+  if (ref $type_definition->{inflate}) {
+    $Moo::HandleMoose::TYPE_MAP{$isa} = $type_definition->{inflate};
+  }
+  elsif (exists $type_definition->{inflate} and not $type_definition->{inflate}) {
+    # no-op
+  }
+  else {
+    my $full_name =
+      defined $moose_namespace 
+      ? "${moose_namespace}::" . $type_definition->{name}
+      : $type_definition->{name};
 
-  $Moo::HandleMoose::TYPE_MAP{$isa} = sub {
-    require_module($moose_namespace) if $moose_namespace;
-    Moose::Util::TypeConstraints::find_type_constraint($full_name);
-    };
+    $Moo::HandleMoose::TYPE_MAP{$isa} = sub {
+      require_module($moose_namespace) if $moose_namespace;
+      Moose::Util::TypeConstraints::find_type_constraint($full_name);
+      };
+  }
 
   return {
     type => sub {
@@ -103,6 +111,11 @@ sub make_type {
           }
           };
 
+        if (ref $type_definition->{inflate}) {
+          my $inflation = $type_definition->{inflate};
+          $Moo::HandleMoose::TYPE_MAP{$parameterized_isa} = sub { $inflation->(\@params) };
+        }
+
         # Remove old $isa, but return the rest of the arguments
         # so any specs defined after 'isa' don't get lost
         shift;
@@ -122,6 +135,16 @@ sub exception_message {
   return "${attribute_value} is not ${type}!";
 }
 
+sub inflate_type {
+  my $coderef = shift;
+  if (my $inflator = $Moo::HandleMoose::TYPE_MAP{$coderef}) {
+    return $inflator->();
+  }
+  return Moose::Meta::TypeConstraint->new(
+    constraint => sub { eval { &$coderef; 1 } }
+  );
+}
+
 1;
 __END__
 
@@ -194,9 +217,11 @@ Matt Phillips (cpan:MATTP) <mattp@cpan.org>
 
 Arthur Axel fREW Schmidt (cpan:FREW) <frioux@gmail.com>
 
+Toby Inkster (cpan:TOBYINK) <tobyink@cpan.org>
+
 =head1 COPYRIGHT
 
-Copyright (c) 2011-2012 the MooX::Types::MooseLike L</AUTHOR> and
+Copyright (c) 2011-2013 the MooX::Types::MooseLike L</AUTHOR> and
  L</CONTRIBUTORS> as listed above.
 
 =head1 LICENSE
